@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: stdfn.c,v 1.30.2.1 2017/02/28 06:51:04 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - stdfn.c */
 
 /*[
@@ -41,18 +37,27 @@ static char *RCSid() { return RCSid("$Id: stdfn.c,v 1.30.2.1 2017/02/28 06:51:04
  * - Lars Hecking
  */
 
+#if defined(_WIN32) && defined(__WATCOMC__)
+#  include <direct.h>
+#endif
+
 #include "stdfn.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 /* the WIN32 API has a Sleep function that does not consume CPU cycles */
-#include <windows.h>
-#ifndef HAVE_DIRENT_H
-#include <io.h> /* _findfirst and _findnext set errno iff they return -1 */
+# define WIN32_LEAN_AND_MEAN
+# include <windows.h>
+# include "term_api.h"
+# include "win/winmain.h"
+# include <io.h> /* _findfirst and _findnext set errno iff they return -1 */
 #endif
+#if defined(OS2) && !defined(__KLIBC__)
+# define INCL_DOSPROFILE
+# include <os2.h>
 #endif
 #ifdef NEED_CEXP
-#include <math.h>
-#include <complex.h>
+# include <math.h>
+# include <complex.h>
 #endif
 
 /*
@@ -173,12 +178,11 @@ strstr(const char *cs, const char *ct)
  * it doesn't really matter on these systems. lh
  */
 
-
 unsigned int
 sleep(unsigned int delay)
 {
 #if defined(MSDOS)
-# if !((defined(__EMX__) || defined(DJGPP))
+# if !defined(DJGPP)
     /* kludge to provide sleep() for msc 5.1 */
     unsigned long time_is_up;
 
@@ -186,7 +190,7 @@ sleep(unsigned int delay)
     while (time(NULL) < time_is_up)
 	/* wait */ ;
 # endif
-#elif defined(WIN32)
+#elif defined(_WIN32)
     Sleep((DWORD) delay * 1000);
 #endif /* MSDOS ... */
 
@@ -196,6 +200,36 @@ sleep(unsigned int delay)
 #endif /* HAVE_SLEEP */
 
 
+#if defined(OS2) && !defined(__KLIBC__)
+
+void usleep(unsigned long microseconds)
+{
+    if (microseconds >= 32000) {
+	/* The granularity of DosSleep is 32ms. */
+	_sleep2(microseconds / 1000);
+    } else {
+	/* For short wait times we just idle. */
+	ULONG ulTmrFreq; /* frequency of the counter */
+	unsigned long long qwTmrTimeStart; /* counter value */
+	unsigned long long qwTmrTimeStop;
+	ULONG ulDelta;
+	APIRET rc;
+	char overflow;
+
+	rc = DosTmrQueryFreq(&ulTmrFreq);
+	ulDelta = ulTmrFreq * (unsigned long long) microseconds / 1000000LL;
+	rc = DosTmrQueryTime((PQWORD)&qwTmrTimeStart);
+	overflow = (qwTmrTimeStart + ulDelta) < qwTmrTimeStart;
+	do {
+	    rc = DosTmrQueryTime((PQWORD)&qwTmrTimeStop);
+	} while (!((!overflow || (qwTmrTimeStop < qwTmrTimeStart))
+	          && (qwTmrTimeStop >= (qwTmrTimeStart + ulDelta))));
+    }
+}
+
+#endif
+
+
 /*
  * Other common functions
  */
@@ -268,8 +302,8 @@ gp_strnicmp(const char *s1, const char *s2, size_t n)
 #endif /* !HAVE_STRNICMP */
 
 
-#ifndef HAVE_STRNLEN    
-size_t 
+#ifndef HAVE_STRNLEN
+size_t
 strnlen(const char *str, size_t n)
 {
     const char * stop = (char *)memchr(str, '\0', n);
@@ -279,7 +313,7 @@ strnlen(const char *str, size_t n)
 
 
 #ifndef HAVE_STRNDUP
-char * 
+char *
 strndup(const char * str, size_t n)
 {
     char * ret = NULL;
@@ -294,7 +328,6 @@ strndup(const char * str, size_t n)
 
 /* Safe, '\0'-terminated version of strncpy()
  * safe_strncpy(dest, src, n), where n = sizeof(dest)
- * This is basically the old fit.c(copy_max) function
  */
 char *
 safe_strncpy(char *d, const char *s, size_t n)
@@ -312,7 +345,7 @@ safe_strncpy(char *d, const char *s, size_t n)
 #ifndef HAVE_STRCSPN
 /*
  * our own substitute for strcspn()
- * return the length of the inital segment of str1
+ * return the length of the initial segment of str1
  * consisting of characters not in str2
  * returns strlen(str1) if none of the characters
  * from str2 are in str1
@@ -336,7 +369,7 @@ gp_strcspn(const char *str1, const char *str2)
 
 
 /* Standard compliant replacement functions for MSVC */
-#if defined(_MSC_VER)
+#if defined(_MSC_VER) && (_MSC_VER < 1900)
 int
 ms_vsnprintf(char *str, size_t size, const char *format, va_list ap)
 {
@@ -365,16 +398,14 @@ ms_snprintf(char *str, size_t size, const char * format, ...)
 
 
 /* Implement portable generation of a NaN value. */
-/* NB: Supposedly DJGPP V2.04 can use atof("NaN"), but... */
 
 double
 not_a_number(void)
 {
-#if defined (__MSC__) || defined (DJGPP) || defined(__DJGPP__) || defined(__MINGW32__)
-	unsigned long lnan[2]={0xffffffff, 0x7fffffff};
-    return *( double* )lnan;
+#ifdef NAN
+    return NAN; /* C99 define */
 #else
-	return atof("NaN");
+    return atof("NaN");
 #endif
 }
 
@@ -382,9 +413,9 @@ not_a_number(void)
 #ifdef NEED_CEXP
 double _Complex cexp(double _Complex z)
 {
-	double x = creal(z);
-	double y = cimag(z);
-	return exp(x) * (cos(y) + I*sin(y));
+    double x = creal(z);
+    double y = cimag(z);
+    return gp_exp(x) * (cos(y) + I*sin(y));
 }
 #endif
 
@@ -485,8 +516,22 @@ void gp_exit(int status)
     exit(status);
 }
 
-#if !defined(HAVE_DIRENT_H) && defined(WIN32)  && (!defined(__WATCOMC__))
-/* BM: OpenWatcom has dirent functions in direct.h!*/
+#ifdef _WIN32
+char *
+gp_getcwd(char *path, size_t len)
+{
+    wchar_t wpath[MAX_PATH + 1];
+    if (_wgetcwd(wpath, MAX_PATH) != NULL) {
+	WideCharToMultiByte(WinGetCodepage(encoding), 0, wpath, -1, path, len, NULL, 0);
+	return path;
+    }
+    return NULL;
+}
+#endif
+
+#ifdef _WIN32
+/* Note: OpenWatcom has dirent functions in direct.h but we use our functions
+		 since they support encodings. */
 /*
 
     Implementation of POSIX directory browsing functions and types for Win32.
@@ -499,88 +544,105 @@ void gp_exit(int status)
 
 struct DIR
 {
-    long                handle; /* -1 for failed rewind */
-    struct _finddata_t  info;
-    struct dirent       result; /* d_name null iff first time */
-    char                *name;  /* null-terminated char string */
+    intptr_t            handle; /* -1 for failed rewind */
+    struct _wfinddata_t info;
+    struct gp_dirent    result; /* d_name null iff first time */
+    WCHAR               *name;  /* null-terminated string */
+    char                info_mbname[4*260];
 };
 
-DIR *opendir(const char *name)
+
+DIR *
+gp_opendir(const char *name)
 {
     DIR *dir = 0;
+    char *mbname;
 
     if (name && name[0]) {
-        size_t base_length = strlen(name);
-         /* search pattern must end with suitable wildcard */
-        const char *all = strchr("/\\", name[base_length - 1]) ? "*" : "/*";
+	size_t base_length = strlen(name);
+	    /* search pattern must end with suitable wildcard */
+	const char *all = strchr("/\\", name[base_length - 1]) ? "*" : "/*";
 
-        if ((dir = (DIR *) malloc(sizeof *dir)) != 0 &&
-           (dir->name = (char *) malloc(base_length + strlen(all) + 1)) != 0) {
-            strcat(strcpy(dir->name, name), all);
+	if ((dir = (DIR *) malloc(sizeof *dir)) != NULL &&
+	    (mbname = (char *) malloc(base_length + strlen(all) + 1)) != NULL) {
+	    strcat(strcpy(mbname, name), all);
+	    dir->name = UnicodeText(mbname, encoding);
+	    free(mbname);
 
-            if ((dir->handle = (long) _findfirst(dir->name, &dir->info)) != -1) {
-                dir->result.d_name = 0;
-            } else { /* rollback */
-                free(dir->name);
-                free(dir);
-                dir = 0;
-            }
-        } else { /* rollback */
-            free(dir);
-            dir   = 0;
-            errno = ENOMEM;
-        }
+	    if ((dir->name != NULL) &&
+		((dir->handle = _wfindfirst(dir->name, &dir->info)) != -1)) {
+		dir->result.d_name = NULL;
+	    } else { /* rollback */
+		free(dir->name);
+		free(dir);
+		dir = NULL;
+	    }
+	} else { /* rollback */
+	    free(dir);
+	    dir   = NULL;
+	    errno = ENOMEM;
+	}
     } else {
-        errno = EINVAL;
+	errno = EINVAL;
     }
 
     return dir;
 }
 
-int closedir(DIR *dir)
+
+int
+gp_closedir(DIR *dir)
 {
     int result = -1;
 
     if (dir) {
-        if(dir->handle != -1) {
-            result = _findclose(dir->handle);
-        }
-        free(dir->name);
-        free(dir);
+	if (dir->handle != -1) {
+	    result = _findclose(dir->handle);
+	}
+	free(dir->name);
+	free(dir);
     }
 
     if (result == -1) { /* map all errors to EBADF */
-        errno = EBADF;
+	errno = EBADF;
     }
 
     return result;
 }
 
-struct dirent *readdir(DIR *dir)
+
+struct gp_dirent *
+gp_readdir(DIR *dir)
 {
-    struct dirent *result = 0;
+    struct gp_dirent *result = 0;
 
     if (dir && dir->handle != -1) {
-        if (!dir->result.d_name || _findnext(dir->handle, &dir->info) != -1) {
-            result         = &dir->result;
-            result->d_name = dir->info.name;
-        }
+	if (!dir->result.d_name || _wfindnext(dir->handle, &dir->info) != -1) {
+	    result         = &dir->result;
+	    WideCharToMultiByte(WinGetCodepage(encoding), 0,
+				dir->info.name, sizeof(dir->info.name) / sizeof(wchar_t),
+				dir->info_mbname, sizeof(dir->info_mbname) / sizeof(char),
+				NULL, 0);
+	    result->d_name = dir->info_mbname;
+	}
     } else {
-        errno = EBADF;
+	errno = EBADF;
     }
 
     return result;
 }
 
-void rewinddir(DIR *dir)
+
+void
+gp_rewinddir(DIR *dir)
 {
     if (dir && dir->handle != -1) {
-        _findclose(dir->handle);
-        dir->handle = (long) _findfirst(dir->name, &dir->info);
-        dir->result.d_name = 0;
+	_findclose(dir->handle);
+	dir->handle = (long) _wfindfirst(dir->name, &dir->info);
+	dir->result.d_name = NULL;
     }
     else {
-        errno = EBADF;
+	errno = EBADF;
     }
 }
 
@@ -592,10 +654,10 @@ void rewinddir(DIR *dir)
     documentation for any purpose is hereby granted without fee, provided
     that this copyright and permissions notice appear in all copies and
     derivatives.
-    
+
     This software is supplied "as is" without express or implied warranty.
 
     But that said, if there are any problems please get in touch.
 
 */
-#endif /* !HAVE_DIRENT_H && WIN32 */
+#endif /* _WIN32 */
